import React from 'react';
import { Locale } from '../locales';
export interface CustomValue<T = Locale> {
    /** Language configuration */
    locale?: T;
    /** Support right-to-left */
    rtl?: boolean;
    /**
     * Date Formatting API
     *
     * Example:
     *
     *  import format from 'date-fns/format';
     *  import ru from 'date-fns/locale/ru';
     *
     *  function formatDate(data, formatStr) {
     *    return format(data, formatStr, { locale: ru });
     *  }
     *
     * */
    formatDate?: (date: Date | string | number, format?: string, options?: {
        locale?: any;
    }) => string;
}
export interface CustomProviderProps<T = any> extends CustomValue<T> {
    /** Supported themes */
    theme?: 'default' | 'dark' | 'high-contrast';
    /** The prefix of the component CSS class */
    classPrefix?: string;
    /** Primary content */
    children?: React.ReactNode;
}
declare const CustomContext: React.Context<CustomProviderProps<any>>;
declare const Consumer: React.Consumer<CustomProviderProps<any>>;
declare const CustomProvider: (props: CustomProviderProps) => JSX.Element;
export { CustomContext, Consumer as CustomConsumer };
export default CustomProvider;
