"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

exports.__esModule = true;
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _DOMHelper = _interopRequireDefault(require("../DOMHelper"));

var _positionUtils = _interopRequireDefault(require("./positionUtils"));

var _utils = require("../utils");

var defaultProps = {
  containerPadding: 0,
  placement: 'right'
};

var usePosition = function usePosition(props, ref) {
  var placement = props.placement,
      preventOverflow = props.preventOverflow,
      containerPadding = props.containerPadding,
      container = props.container,
      triggerTarget = props.triggerTarget;
  var containerRef = (0, _react.useRef)();
  var containerScrollListenerRef = (0, _react.useRef)();
  var lastTargetRef = (0, _react.useRef)();
  var defaultPosition = {
    positionLeft: 0,
    positionTop: 0,
    arrowOffsetLeft: null,
    arrowOffsetTop: null
  };

  var _useState = (0, _react.useState)(defaultPosition),
      position = _useState[0],
      setPosition = _useState[1];

  var utils = (0, _react.useMemo)(function () {
    return (0, _positionUtils.default)({
      placement: placement,
      preventOverflow: preventOverflow,
      padding: containerPadding
    });
  }, [placement, preventOverflow, containerPadding]);
  var updatePosition = (0, _react.useCallback)(function (placementChanged) {
    if (placementChanged === void 0) {
      placementChanged = true;
    }

    if (!(triggerTarget === null || triggerTarget === void 0 ? void 0 : triggerTarget.current)) {
      return;
    }

    var targetElement = (0, _utils.getDOMNode)(triggerTarget);

    if (!_DOMHelper.default.isElement(targetElement)) {
      throw new Error('`target` should return an HTMLElement');
    }
    /**
     * If the target and placement do not change, the position is not updated.
     */


    if (targetElement === lastTargetRef.current && !placementChanged) {
      return;
    }

    var overlay = (0, _utils.getDOMNode)(ref.current);

    var containerElement = _DOMHelper.default.getContainer(typeof container === 'function' ? container() : container, _DOMHelper.default.ownerDocument(ref.current).body);

    setPosition(utils.calcOverlayPosition(overlay, targetElement, containerElement));
    containerRef.current = containerElement;
    lastTargetRef.current = targetElement;
  }, [container, ref, triggerTarget, utils]);
  (0, _react.useEffect)(function () {
    updatePosition(false);

    if (containerRef.current && preventOverflow) {
      var _containerRef$current;

      containerScrollListenerRef.current = _DOMHelper.default.on(((_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.tagName) === 'BODY' ? window : containerRef.current, 'scroll', function () {
        return updatePosition();
      });
    }

    return function () {
      var _containerScrollListe;

      lastTargetRef.current = null;
      (_containerScrollListe = containerScrollListenerRef.current) === null || _containerScrollListe === void 0 ? void 0 : _containerScrollListe.off();
    };
  }, [preventOverflow, updatePosition]);
  (0, _react.useEffect)(function () {
    return updatePosition();
  }, [updatePosition, placement]);
  return [position, updatePosition];
};

var Position = /*#__PURE__*/_react.default.forwardRef(function (props, ref) {
  var children = props.children,
      className = props.className;

  var childRef = _react.default.useRef();

  var _usePosition = usePosition(props, childRef),
      position = _usePosition[0],
      updatePosition = _usePosition[1];

  var positionClassName = position.positionClassName,
      arrowOffsetLeft = position.arrowOffsetLeft,
      arrowOffsetTop = position.arrowOffsetTop,
      positionLeft = position.positionLeft,
      positionTop = position.positionTop;
  (0, _react.useImperativeHandle)(ref, function () {
    return {
      get child() {
        return childRef.current;
      },

      updatePosition: updatePosition
    };
  });

  if (typeof children === 'function') {
    var childProps = {
      className: (0, _classnames.default)(className, positionClassName),
      arrowOffsetLeft: arrowOffsetLeft,
      arrowOffsetTop: arrowOffsetTop,
      left: positionLeft,
      top: positionTop
    };
    return children(childProps, childRef);
  }

  return children;
});

Position.displayName = 'Position';
Position.defaultProps = defaultProps;
Position.propTypes = {
  className: _propTypes.default.string,
  children: _propTypes.default.func,
  container: _propTypes.default.oneOfType([_propTypes.default.func, _propTypes.default.any]),
  containerPadding: _propTypes.default.number,
  placement: _propTypes.default.any,
  preventOverflow: _propTypes.default.bool,
  triggerTarget: _propTypes.default.any
};
var _default = Position;
exports.default = _default;